/**
 * 函数节流
 * @param {*} fn 事件回调
 * @param {*} interval 时间间隔的阈值
 */
export const throttle = function (fn, interval) {
  let last = 0
  return function () {
    // eslint-disable-next-line no-invalid-this
    const context = this
    const args = arguments
    const now = Number(new Date())

    if (now - last >= interval) {
      last = now
      fn.apply(context, args)
    }
  }
}

/**
 * 函数防抖
 * @param {callback} fn 事件回调
 * @param {number} delay 每次推迟执行的等待时间
 */
export const debounce = function (fn, delay) {
  let last = 0
  let timer = null
  return function () {
    // eslint-disable-next-line no-invalid-this
    const context = this
    const args = arguments
    const now = Number(new Date())

    if (now - last < delay) {
      clearTimeout(timer)
      timer = setTimeout(function () {
        last = now
        fn.apply(context, args)
      }, delay)
    } else {
      last = now
      fn.apply(context, args)
    }
  }
}

/**
 * 深拷贝
 * @param {Object} source
 */
export function deepClone(source) {
  if (!source && typeof source !== 'object') {
    throw new Error('error arguments', 'deepClone')
  }
  const targetObj = source.constructor === Array ? [] : {}
  Object.keys(source).forEach((keys) => {
    if (source[keys] && typeof source[keys] === 'object') {
      targetObj[keys] = deepClone(source[keys])
    } else {
      targetObj[keys] = source[keys]
    }
  })
  return targetObj
}

/**
 * 获取链接某个参数 search
 * @param {String} name 参数名称
 * @returns {String} 返回参数值
 */
export function getQueryString(name) {
  let reg = new RegExp('(^|&)' + name + '=([^&]*)(&|$)', 'i')
  let r = window.location.search.substr(1).match(reg)
  // eslint-disable-next-line eqeqeq, no-eq-null
  if (r != null) {
    return decodeURIComponent(r[2])
  }
  return null
}

/**
 * 获取网址参数 search 和 hash
 * @returns {Object} 返回包含当前URL参数的对象
 *
 */
export function getURLParameters() {
  const reg = /([^?=&]+)(=([^&]*))/g
  const searchParamList = window.location.search.match(reg)
  const hashParamList = window.location.hash.match(reg)
  const obj = {}

  searchParamList &&
    searchParamList.forEach((v) => {
      obj[v.slice(0, v.indexOf('='))] = decodeURIComponent(v.slice(v.indexOf('=') + 1))
    })

  hashParamList &&
    hashParamList.forEach((v) => {
      obj[v.slice(0, v.indexOf('='))] = decodeURIComponent(v.slice(v.indexOf('=') + 1))
    })

  return obj
}
/**
 * getEnv
 * @returns {} 环境信息
 */
export function getEnv() {
  const ua = navigator.userAgent.toLowerCase() // 一律小写
  const isAndroid = /android/i.test(ua)
  const isIOS = /iphone|ipad|ipod|ios/i.test(ua)
  const isInMiniProgram = /miniProgram/i.test(ua) // 微信小程序内
  const isInWeixinApp = /micromessenger/i.test(ua) // 微信内
  const isInWeChatDevTools = /wechatdevtools/i.test(ua) // 微信开发者工具内

  return {
    isAndroid,
    isIOS,
    isInMiniProgram,
    isInWeixinApp,
    isInWeChatDevTools,
  }
}

/**
 * 获取接口前缀
 * @param {*} code api, host, origin
 * @returns
 */
export function getAPI(code = 'api') {
  const host = process.env.NODE_ENV === 'production' ? location.host : process.env.VUE_APP_BASE_HOST
  const basePath = process.env.NODE_ENV === 'production' ? `/${location.pathname.split('/')[1]}` : '/super_cloud' // 基础部署路径
  const origin = `${location.protocol}//${host}`
  const api = `${origin}${basePath}`

  switch (code) {
    case 'host':
      return host
    case 'origin':
      return origin
    default:
      return api
  }
}
/**
 * idDefString
 * @param {*} str
 */
export function idDefString(str) {
  return str !== 'undefined' && str !== 'null'
}
/**
 * 获取服务之窗的缓存
 * @param {*} key
 * @param {*} def
 * @returns
 */
export function getLocalforage(key, def) {
  const str = localStorage.getItem(`localforage/${key}`)
  if (str) {
    try {
      return idDefString(JSON.parse(str)) ? JSON.parse(str) : ''
    } catch (e) {
      return str || null
    }
  } else {
    // eslint-disable-next-line eqeqeq
    return def != undefined ? def : null
  }
}

/**
 * 设置服务之窗的缓存
 * @param {*} key
 * @param {*} data
 * @returns
 */
export function setLocalforage(key, data) {
  localStorage.setItem(`localforage/${key}`, JSON.stringify(data))
}

/**
 * 生成版本号（时间戳 按5分钟）
 * @returns {String}
 */
export function getTimeStampVersion() {
  const now = new Date()
  const year = now.getFullYear()
  const month = now.getMonth() + 1
  const date = now.getDate()
  const hour = now.getHours()
  const minutes = now.getMinutes()
  const padZero = (n) => String(n).padStart(2, '0')
  const base = 5

  return `${year}${padZero(month)}${padZero(date)}${padZero(hour)}${padZero(base * Math.floor(minutes / base))}`
}

/**
 * 给url 追加参数
 * @param {*} url
 * @param {Object} query
 * @description 多加了个版本号_v，避免缓存
 * @returns {String}
 */
export function interceptorUrl(url, query = {}) {
  const search = url.split('#')[0]
  const hash = url.split('#')[1]
  const params = {
    ...query,
    _v: getTimeStampVersion(),
  }
  // eslint-disable-next-line no-implicit-coercion
  let link = `${search}${~search.indexOf('?') ? '&' : '?'}`

  link += Object.keys(params)
    .map((key) => key + '=' + encodeURIComponent(params[key]))
    .join('&')

  if (hash) {
    link += `#${hash}`
  }

  return link
}

/**
 * rpx2px
 * @param {Number} val
 * @param {Number} destWidth
 */
export function rpx2px(val, destWidth = 375) {
  const scale = document.documentElement.clientWidth / destWidth

  return Math.round(val * scale)
}

/**
 * 返回服务之窗的首页
 */
export function jumpSuperDeskHome() {
  // 小程序环境
  if (getEnv().isInMiniProgram) {
    wx.miniProgram.switchTab({
      url: '/pages/home/index',
    })
  } else {
    let homeUrl = `${window.location.origin}/super_cloud/wx/main?openid=${getLocalforage(
      'openId',
    )}&state=${getLocalforage('pid')}`
    // 微信网页环境
    if (process.env.NODE_ENV === 'development') {
      alert(`返回服务之窗首页${homeUrl}`)
      return
    }
    // 旧 其他版本服务之窗
    if (sessionStorage.getItem('newHomeUrl')) {
      homeUrl = sessionStorage.getItem('newHomeUrl')
      console.log('homeUrl', homeUrl)
    }
    // 返回服务之窗首页
    location.replace(homeUrl)
  }
}

/**
 * 转换驼峰拼写的字符串为特定格式
 * @description 使用 String.replace() 去除下划线，连字符和空格，并将驼峰拼写格式的单词转换为全小写。省略第二个参数 separator ，默认使用_分隔符。
 * @param {Number} n
 */
export const fromCamelCase = function (str, separator = '_') {
  return str
    .replace(/([a-z\d])([A-Z])/g, '$1' + separator + '$2')
    .replace(/([A-Z]+)([A-Z][a-z\d]+)/g, '$1' + separator + '$2')
    .toLowerCase()
}

/**
 * 追加style
 * @param {*} css
 * @param {*} cssId
 * @param {*} replace
 */
export function appendCss(css, cssId, replace = false) {
  let style = document.getElementById(cssId)
  if (style && replace) {
    style.parentNode.removeChild(style)
    style = null
  }
  if (!style) {
    style = document.createElement('style')
    style.type = 'text/css'
    cssId && (style.id = cssId)
    document.getElementsByTagName('head')[0].appendChild(style)
  }
  style.appendChild(document.createTextNode(css))
}
